package com.paymentz.checkout_plugin;

import android.app.Application;
import android.content.Intent;
import android.os.Bundle;
import android.app.Activity;

import androidx.annotation.NonNull;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.DefaultLifecycleObserver;

import androidx.lifecycle.LifecycleOwner;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import com.paymentz.model.EncryptPay;

import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;

import com.paymentz.model.RequestParameters;
import com.google.gson.Gson;

import io.flutter.embedding.engine.plugins.activity.ActivityAware;

import java.util.HashMap;

import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.embedding.engine.plugins.lifecycle.FlutterLifecycleAdapter;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;

import static android.app.Activity.RESULT_OK;

/**
 * CheckoutPlugin
 */
public class CheckoutPlugin implements FlutterPlugin, PluginRegistry.ActivityResultListener, MethodCallHandler, ActivityAware {

    private static FlutterActivity activity;
    private static Result pendingResult;
    private MethodChannel channel;
    private FlutterPluginBinding pluginBinding;
    private ActivityPluginBinding activityBinding;
    private Application applicationContext;
    private Lifecycle lifecycle;
    private LifeCycleObserver observer;

    RequestParameters requestParameters = new RequestParameters();
    PluginRegistry.Registrar pluginRegistrar;
    HashMap<String, String> acctyp = new HashMap<String, String>();
    String memberId, paymentMode, terminalId, secureKey, merchantTransactionId, amount, currency, toType, paymentBrand, merchantRedirectUrl, tmplAmount, tmplCurrency, orderDescription, country, state, street, city, email, postCode, telnocc, hostUrl, phone;


    public CheckoutPlugin() {
    }

    private CheckoutPlugin(FlutterActivity activity, final PluginRegistry.Registrar registrar) {
        CheckoutPlugin.activity = activity;
    }

    public static void registerWith(final PluginRegistry.Registrar registrar) {
        if (registrar.activity() == null) {
            return;
        }
        Activity activity = registrar.activity();
        Application applicationContext = null;
        if (registrar.context() != null) {
            applicationContext = (Application) (registrar.context().getApplicationContext());
        }
        CheckoutPlugin instance = new CheckoutPlugin((FlutterActivity) registrar.activity(), registrar);
        instance.createPluginSetup(registrar.messenger(), applicationContext, activity, registrar, null);
    }


    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        this.pendingResult = result;
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else if (call.method.equals("payment")) {
            //Toast.makeText(applicationContext, "Hello Fluter.", Toast.LENGTH_SHORT).show();
            acctyp = call.argument("requestParameters");
            secureKey = call.argument("secureKey");
            memberId = acctyp.get("memberId"); //done
            paymentMode = acctyp.get("paymentMode"); //done
            terminalId = acctyp.get("terminalId"); //done
            merchantTransactionId = acctyp.get("merchantTransactionId"); //done
            amount = acctyp.get("amount"); //done
            currency = acctyp.get("currency");
            toType = acctyp.get("toType");
            paymentBrand = acctyp.get("paymentBrand");
            merchantRedirectUrl = acctyp.get("merchantRedirectUrl");
            tmplAmount = acctyp.get("tmplAmount");
            tmplCurrency = acctyp.get("tmplCurrency");
            orderDescription = acctyp.get("orderDescription");
            country = acctyp.get("country");
            state = acctyp.get("state");
            street = acctyp.get("street");
            city = acctyp.get("city");
            email = acctyp.get("email");
            postCode = acctyp.get("postCode");
            telnocc = acctyp.get("telnocc");
            phone = acctyp.get("phone");
            hostUrl = acctyp.get("hostUrl");
            requestParameters.setTmplAmount(tmplAmount);
            requestParameters.setAmount(amount);
            requestParameters.setMerchantTransactionId(merchantTransactionId);
            requestParameters.setMemberId(memberId);
            requestParameters.setToType(toType);
            requestParameters.setMemberKey(secureKey);
            requestParameters.setOrderDescription(orderDescription);
            requestParameters.setMerchantRedirectUrl(merchantRedirectUrl);
            requestParameters.setCountry(country);
            requestParameters.setCity(city);
            requestParameters.setState(state);
            requestParameters.setPostCode(postCode);
            requestParameters.setStreet(street);
            requestParameters.setTelnocc(telnocc);
            requestParameters.setPhone(phone);
            requestParameters.setEmail(email);
            requestParameters.setIpAddress("");
            requestParameters.setPaymentBrand(paymentBrand);
            requestParameters.setPaymentMode(paymentMode);
            requestParameters.setCurrency(currency);
            requestParameters.setTmplCurrency(tmplCurrency);
            requestParameters.setTerminalId(terminalId);
            requestParameters.setVersion("1.0");
            requestParameters.setHostUrl(hostUrl);
            EncryptPay.initPayment(activity, requestParameters);
        } else {
            result.notImplemented();
        }
    }


    @Override
    public void onAttachedToEngine(FlutterPlugin.FlutterPluginBinding binding) {
        pluginBinding = binding;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        pluginBinding = null;
    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        activityBinding = binding;
        createPluginSetup(
                pluginBinding.getBinaryMessenger(),
                (Application) pluginBinding.getApplicationContext(),
                activityBinding.getActivity(),
                null,
                activityBinding);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        onDetachedFromActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        clearPluginSetup();
    }

    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == EncryptPay.PAYMENT_REQUEST_CODE) {
            if (resultCode == RESULT_OK) {
                String paymentResult = data.getStringExtra(EncryptPay.PAYMENT_RESULT);
                HashMap<String, Object> map = new Gson().fromJson(paymentResult, HashMap.class);
                pendingResult.success(map);
            }
        }
        return false;
    }


    private class LifeCycleObserver
            implements Application.ActivityLifecycleCallbacks, DefaultLifecycleObserver {
        private final Activity thisActivity;

        LifeCycleObserver(Activity activity) {
            this.thisActivity = activity;
        }

        @Override
        public void onCreate(@NonNull LifecycleOwner owner) {
        }

        @Override
        public void onStart(@NonNull LifecycleOwner owner) {
        }

        @Override
        public void onResume(@NonNull LifecycleOwner owner) {
        }

        @Override
        public void onPause(@NonNull LifecycleOwner owner) {
        }

        @Override
        public void onStop(@NonNull LifecycleOwner owner) {
            onActivityStopped(thisActivity);
        }

        @Override
        public void onDestroy(@NonNull LifecycleOwner owner) {
            onActivityDestroyed(thisActivity);
        }

        @Override
        public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
        }

        @Override
        public void onActivityStarted(Activity activity) {
        }

        @Override
        public void onActivityResumed(Activity activity) {
        }

        @Override
        public void onActivityPaused(Activity activity) {
        }

        @Override
        public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
        }

        @Override
        public void onActivityDestroyed(Activity activity) {
            if (thisActivity == activity && activity.getApplicationContext() != null) {
                ((Application) activity.getApplicationContext())
                        .unregisterActivityLifecycleCallbacks(
                                this);
            }
        }

        @Override
        public void onActivityStopped(Activity activity) {

        }
    }


    private void clearPluginSetup() {
        activity = null;
        activityBinding.removeActivityResultListener(this);
        activityBinding = null;
        lifecycle.removeObserver(observer);
        lifecycle = null;
        channel.setMethodCallHandler(null);
        channel = null;
        applicationContext.unregisterActivityLifecycleCallbacks(observer);
        applicationContext = null;
    }

    private void createPluginSetup(
            final BinaryMessenger messenger,
            final Application applicationContext,
            final Activity activity,
            final PluginRegistry.Registrar registrar,
            final ActivityPluginBinding activityBinding) {


        this.activity = (FlutterActivity) activity;


        this.applicationContext = applicationContext;
        channel = new MethodChannel(messenger, "checkout_plugin");
        channel.setMethodCallHandler(this);
        if (registrar != null) {
            System.out.println("registrar");
            // V1 embedding setup for activity listeners.
            observer = new LifeCycleObserver(activity);
            applicationContext.registerActivityLifecycleCallbacks(
                    observer); // Use getApplicationContext() to avoid casting failures.
            registrar.addActivityResultListener(this);
        } else {
            System.out.println("V2 embedding setup for activity listeners.");
            // V2 embedding setup for activity listeners.
            activityBinding.addActivityResultListener(this);
            lifecycle = FlutterLifecycleAdapter.getActivityLifecycle(activityBinding);
            observer = new LifeCycleObserver(activity);
            lifecycle.addObserver(observer);
        }
    }
}
