//
//  StandardCheckoutViewController.swift
//  StandardCheckoutKit
//
//  Created by Vinicius on 09/06/17.
//  Copyright © 2017 Paymentz. All rights reserved.
//

import UIKit
import WebKit
import Flutter

public protocol StandardCheckoutDelegate {
    func onSuccess(standardCheckoutResult: StandardCheckoutResult)
    func onFail(response:[String:Any])
}

class StandardCheckoutViewController: UIViewController, WKScriptMessageHandler, WKNavigationDelegate, WKUIDelegate {

    public var requestParameters: RequestParameters?
    public var secureKey = ""
    public var standardCheckoutDelegate: StandardCheckoutDelegate?
    
    private var webView: WKWebView!
    private let activityIndicator = UIActivityIndicatorView(style: .gray)
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.title = "Payment"
        let preferences = WKPreferences()
        preferences.javaScriptEnabled = true
        let config = WKWebViewConfiguration()
        config.preferences = preferences
        let contentController = WKUserContentController()
        contentController.add(self, name: "callbackHandler")
        config.userContentController = contentController
        webView = WKWebView(frame: self.view.bounds, configuration: config)
        
        if requestParameters != nil {
            
            let dataForMd5Hash = requestParameters!.memberId!+"|"+requestParameters!.toType!+"|"+requestParameters!.amount!+"|"+requestParameters!.merchantTransactionId!+"|"+requestParameters!.merchantRedirectUrl!+"|"+secureKey
            
            requestParameters!.checksum = StandardCheckout.generateMD5ChecksumDirectKit(directKitStringFormat: dataForMd5Hash)
            
            var parameters = "version=1.0_MiOS&checksum="+requestParameters!.checksum!
            
            parameters += "&memberId="+requestParameters!.memberId!
                + "&totype="+requestParameters!.toType!
                + "&merchantTransactionId="+requestParameters!.merchantTransactionId!
                + "&amount="+requestParameters!.amount!
                + "&TMPL_AMOUNT="+requestParameters!.tmplAmount!
                + "&orderDescription="+requestParameters!.orderDescription!
                + "&merchantRedirectUrl="+requestParameters!.merchantRedirectUrl!
                + "&country="+requestParameters!.country!
                + "&city="+requestParameters!.city!
                + "&state="+requestParameters!.state!
                + "&postcode="+requestParameters!.postCode!
                + "&street="+requestParameters!.street!
                + "&telnocc="+requestParameters!.telnocc!
                + "&phone="+requestParameters!.phone!
                + "&email="+requestParameters!.email!
                + "&currency="+requestParameters!.currency!
                + "&TMPL_CURRENCY="+requestParameters!.tmplCurrency!
                + "&terminalid="+requestParameters!.terminalId!
                + "&paymentMode="+requestParameters!.paymentMode!
                + "&paymentBrand="+requestParameters!.paymentBrand!
                + "&device="+requestParameters!.device!
            
            var request = URLRequest(url: URL(string: "\(requestParameters!.hostUrl ?? "https://preprod.paymentz.com/transaction/Checkout")" + "?" + parameters)!)
            request.httpMethod = "POST"
            webView.uiDelegate = self
            webView.navigationDelegate = self
            self.view = webView
            webView.load(request)
            
            activityIndicator.center = self.view.center
            
            self.view.addSubview(activityIndicator)
            
            activityIndicator.startAnimating()
        }
        
        let navBar = UINavigationBar(frame: CGRect(x: 0, y: 0, width: view.frame.size.width, height: 44))
        view.addSubview(navBar)

        let navItem = UINavigationItem(title: "Payment")
        let doneItem = UIBarButtonItem(barButtonSystemItem: .cancel, target: nil, action: #selector(self.cancelClicked))
        //navItem.rightBarButtonItem = doneItem
        navItem.leftBarButtonItem = doneItem
        navBar.setItems([navItem], animated: false)
    }
    @objc func cancelClicked()
    {
        self.dismiss(animated: true) { [self] in
            let response = ["resultCode": "BACK",
                            "message": "BACK PRESSED."]
            standardCheckoutDelegate?.onFail(response: response)
        }
    }

    func webView(_ webView: WKWebView, runJavaScriptAlertPanelWithMessage message: String, initiatedByFrame frame: WKFrameInfo, completionHandler: @escaping () -> Void) {
        
        let alertController = UIAlertController(title: message, message: nil, preferredStyle: UIAlertController.Style.alert)
        alertController.addAction(UIAlertAction(title: "OK", style: .cancel, handler: {(action) in
            completionHandler()}))
        present(alertController, animated: true, completion: nil)
    }
    
    func userContentController(_ userContentController: WKUserContentController, didReceive message: WKScriptMessage) {
        
        print("Message ==  \(message.body) \(message.name)")
        
        if message.name == "callbackHandler" {
            let body = message.body
            self.dismiss(animated: true) { [self] in
                standardCheckoutDelegate?.onSuccess(standardCheckoutResult: StandardCheckoutResult(body as! [String : Any]))
            }
        } else {
            
            self.dismiss(animated: true) { [self] in
                let body = message.body as! [String : Any]
                standardCheckoutDelegate?.onFail(response: body)
            }
        }
    }
    
    // Web view methods
    func webView(_ webView: WKWebView, didFail navigation: WKNavigation!, withError error: Error) {
        activityIndicator.stopAnimating()
        print(error)
    }
    
    func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
        activityIndicator.stopAnimating()
    }
}
