import 'dart:async';

import 'package:flutter/services.dart';

import 'package:eventify/eventify.dart';

class CheckoutPlugin {
  static const _SUCCESS = '00001';
  static const _BACK_PRESSED = 'BACK';

  static const EVENT_SUCCESS = "OnSuccess";
  static const EVENT_FAILED = "OnFail";

  late EventEmitter _eventEmitter;
  static const MethodChannel _channel = const MethodChannel('checkout_plugin');

  CheckoutPlugin() {
    _eventEmitter = new EventEmitter();
  }

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  void payment(Map<String, dynamic> parameter) async {
    var response = await _channel.invokeMethod('payment', parameter);
    _handleResult(response);
  }

  /// Registers event listeners for payment events
  void on(String event, Function handler) {
    EventCallback cb = (event, cont) {
      handler(event.eventData);
    };
    _eventEmitter.on(event, null, cb);
    _resync();
  }

  /// Handles checkout response from platform
  void _handleResult(Map<dynamic, dynamic> response) {
    String eventName;
    dynamic payload;

    switch (response['resultCode'].toString()) {
      case _SUCCESS:
        eventName = EVENT_SUCCESS;
        payload = PaymentSuccessResponse.fromMap(response);
        break;

      case _BACK_PRESSED:
        Map map = {'resultCode': 'BACK', 'message': 'Back Pressed.'};
        eventName = EVENT_FAILED;
        payload = PaymentFailureResponse.fromMap(map);
        break;

      default:
        eventName = EVENT_FAILED;
        payload = PaymentFailureResponse.fromMap(response);
        break;
    }

    _eventEmitter.emit(eventName, null, payload);
    clear();
  }

  void clear() {
    _eventEmitter.clear();
  }

  /// Retrieves lost responses from platform
  void _resync() async {
    var response = await _channel.invokeMethod('resync');
    if (response != null) {
      _handleResult(response);
    }
  }
}

class PaymentSuccessResponse {
  Map response;

  PaymentSuccessResponse(this.response);

  static PaymentSuccessResponse fromMap(Map<dynamic, dynamic> map) {
    Map response = map;

    return new PaymentSuccessResponse(response);
  }
}

class PaymentFailureResponse {
  Map response;

  PaymentFailureResponse(this.response);

  static PaymentFailureResponse fromMap(Map<dynamic, dynamic> map) {
    Map response = map;
    return new PaymentFailureResponse(response);
  }
}
